@@ -5,11 +5,12 @@ from datetime import datetime
 import time
 import threading
 import subprocess
+import json
 
 from croniter import croniter
 
 from dagobah.core.dag import DAG
-from dagobah.core.components import Scheduler, JobState
+from dagobah.core.components import Scheduler, JobState, StrictJSONEncoder
 from dagobah.backend.base import BaseBackend
 
 
@@ -60,38 +61,63 @@ class Dagobah(object):
 
     def from_backend(self, dagobah_id):
         """ Reconstruct this Dagobah instance from the backend. """
-
         rec = self.backend.get_dagobah_json(dagobah_id)
         if not rec:
-            raise DagobahError('dagobah with id %s does not exist in backend' %
-                               dagobah_id)
+            raise DagobahError('dagobah with id %s does not exist '
+                               'in backend' % dagobah_id)
+        self._construct_from_json(rec)
+
+
+    def _construct_from_json(self, rec):
+        """ Construct this Dagobah instance from a JSON document. """
 
-        # delete current version of this Dagobah instance
         self.delete()
 
         for required_key in ['dagobah_id', 'created_jobs']:
             setattr(self, required_key, rec[required_key])
 
         for job_json in rec.get('jobs', []):
-
-            self.add_job(str(job_json['name']), job_json['job_id'])
-            job = self.get_job(job_json['name'])
-            if job_json.get('cron_schedule', None):
-                job.schedule(job_json['cron_schedule'])
-
-            for task in job_json.get('tasks', []):
-                self.add_task_to_job(job,
-                                     str(task['command']),
-                                     str(task['name']))
-
-            dependencies = job_json.get('dependencies', {})
-            for from_node, to_nodes in dependencies.iteritems():
-                for to_node in to_nodes:
-                    job.add_dependency(from_node, to_node)
+            self._add_job_from_spec(job_json)
 
         self.commit(cascade=True)
 
 
+    def add_job_from_json(self, job_json, destructive=False):
+        """ Construct a new Job from an imported JSON spec. """
+        rec = self.backend.decode_import_json(job_json)
+        if destructive:
+            try:
+                dagobah.delete_job(rec['name'])
+            except:
+                pass
+        self._add_job_from_spec(rec, use_job_id=False)
+
+
+    def _add_job_from_spec(self, job_json, use_job_id=True):
+        """ Add a single job to the Dagobah from a spec. """
+
+        job_id = (job_json['job_id']
+                  if use_job_id
+                  else self.backend.get_new_job_id())
+        self.add_job(str(job_json['name']), job_id)
+        job = self.get_job(job_json['name'])
+        if job_json.get('cron_schedule', None):
+            job.schedule(job_json['cron_schedule'])
+
+        for task in job_json.get('tasks', []):
+            self.add_task_to_job(job,
+                                 str(task['command']),
+                                 str(task['name']),
+                                 soft_timeout=task.get('soft_timeout', 0),
+                                 hard_timeout=task.get('hard_timeout', 0))
+
+        dependencies = job_json.get('dependencies', {})
+        for from_node, to_nodes in dependencies.iteritems():
+            for to_node in to_nodes:
+                job.add_dependency(from_node, to_node)
+
+
+
     def commit(self, cascade=False):
         """ Commit this Dagobah instance to the backend.
 
@@ -142,11 +168,13 @@ class Dagobah(object):
             if job.name == job_name:
                 self.backend.delete_job(job.job_id)
                 del self.jobs[idx]
+                self.commit()
                 return
         raise DagobahError('no job with name %s exists' % job_name)
 
 
-    def add_task_to_job(self, job_or_job_name, task_command, task_name=None):
+    def add_task_to_job(self, job_or_job_name, task_command, task_name=None,
+                        **kwargs):
         """ Add a task to a job owned by the Dagobah instance. """
 
         if isinstance(job_or_job_name, Job):
@@ -161,7 +189,7 @@ class Dagobah(object):
             raise DagobahError("job's graph is immutable in its current state: %s"
                                % job.state.status)
 
-        job.add_task(task_command, task_name)
+        job.add_task(task_command, task_name, **kwargs)
         job.commit()
 
 
@@ -172,12 +200,16 @@ class Dagobah(object):
                 else True)
 
 
-    def _serialize(self, include_run_logs=False):
+    def _serialize(self, include_run_logs=False, strict_json=False):
         """ Serialize a representation of this Dagobah object to JSON. """
-        return {'dagobah_id': self.dagobah_id,
-                'created_jobs': self.created_jobs,
-                'jobs': [job._serialize(include_run_logs=include_run_logs)
-                         for job in self.jobs]}
+        result = {'dagobah_id': self.dagobah_id,
+                  'created_jobs': self.created_jobs,
+                  'jobs': [job._serialize(include_run_logs=include_run_logs,
+                                          strict_json=strict_json)
+                           for job in self.jobs]}
+        if strict_json:
+            result = json.loads(json.dumps(result, cls=StrictJSONEncoder))
+        return result
 
 
 class Job(DAG):
@@ -221,7 +253,7 @@ class Job(DAG):
         self.parent.commit()
 
 
-    def add_task(self, command, name=None):
+    def add_task(self, command, name=None, **kwargs):
         """ Adds a new Task to the graph with no edges. """
 
         if not self.state.allow_change_graph:
@@ -230,7 +262,7 @@ class Job(DAG):
 
         if name is None:
             name = command
-        new_task = Task(self, command, name)
+        new_task = Task(self, command, name, **kwargs)
         self.tasks[name] = new_task
         self.add_node(name)
         self.commit()
@@ -283,7 +315,7 @@ class Job(DAG):
         if cron_schedule is None:
             self.cron_schedule = None
             self.cron_iter = None
-            self.next_run =None
+            self.next_run = None
 
         else:
             if base_datetime is None:
@@ -409,6 +441,12 @@ class Job(DAG):
             if key in kwargs and isinstance(kwargs[key], str):
                 setattr(task, key, kwargs[key])
 
+        if 'soft_timeout' in kwargs:
+            task.set_soft_timeout(kwargs['soft_timeout'])
+
+        if 'hard_timeout' in kwargs:
+            task.set_hard_timeout(kwargs['hard_timeout'])
+
         if 'name' in kwargs and isinstance(kwargs['name'], str):
             self.rename_edges(task_name, kwargs['name'])
             self.tasks[kwargs['name']] = task
@@ -524,28 +562,34 @@ class Job(DAG):
         self.backend.commit_log(self.run_log)
 
 
-    def _serialize(self, include_run_logs=False):
+    def _serialize(self, include_run_logs=False, strict_json=False):
         """ Serialize a representation of this Job to a Python dict object. """
 
         # return tasks in sorted order if graph is in a valid state
         try:
             topo_sorted = self._topological_sort()
-            t = [self.tasks[task]._serialize(include_run_logs=include_run_logs)
+            t = [self.tasks[task]._serialize(include_run_logs=include_run_logs,
+                                             strict_json=strict_json)
                  for task in topo_sorted]
         except:
-            t = [task._serialize(include_run_logs=include_run_logs)
+            t = [task._serialize(include_run_logs=include_run_logs,
+                                 strict_json=strict_json)
                  for task in self.tasks.itervalues()]
 
-        return {'job_id': self.job_id,
-                'name': self.name,
-                'parent_id': self.parent.dagobah_id,
-                'tasks': t,
-                'dependencies': {k: list(v)
-                                 for k, v
-                                 in self.graph.iteritems()},
-                'status': self.state.status,
-                'cron_schedule': self.cron_schedule,
-                'next_run': self.next_run}
+        result = {'job_id': self.job_id,
+                  'name': self.name,
+                  'parent_id': self.parent.dagobah_id,
+                  'tasks': t,
+                  'dependencies': {k: list(v)
+                                   for k, v
+                                   in self.graph.iteritems()},
+                  'status': self.state.status,
+                  'cron_schedule': self.cron_schedule,
+                  'next_run': self.next_run}
+
+        if strict_json:
+            result = json.loads(json.dumps(result, cls=StrictJSONEncoder))
+        return result
 
 
 class Task(object):
@@ -556,7 +600,8 @@ class Task(object):
     current serialization of the task with run logs.
     """
 
-    def __init__(self, parent_job, command, name):
+    def __init__(self, parent_job, command, name,
+                 soft_timeout=0, hard_timeout=0):
         self.parent_job = parent_job
         self.backend = self.parent_job.backend
         self.event_handler = self.parent_job.event_handler
@@ -575,6 +620,25 @@ class Task(object):
         self.completed_at = None
         self.successful = None
 
+        self.terminate_sent = False
+        self.kill_sent = False
+
+        self.set_soft_timeout(soft_timeout)
+        self.set_hard_timeout(hard_timeout)
+
+        self.parent_job.commit()
+
+    def set_soft_timeout(self, timeout):
+        if not isinstance(timeout, (int, float)) or timeout < 0:
+            raise ValueError('timeouts must be non-negative numbers')
+        self.soft_timeout = timeout
+        self.parent_job.commit()
+
+    def set_hard_timeout(self, timeout):
+        if not isinstance(timeout, (int, float)) or timeout < 0:
+            raise ValueError('timeouts must be non-negative numbers')
+        self.hard_timeout = timeout
+        self.parent_job.commit()
 
     def reset(self):
         """ Reset this Task to a clean state prior to execution. """
@@ -586,6 +650,9 @@ class Task(object):
         self.completed_at = None
         self.successful = None
 
+        self.terminate_sent = False
+        self.kill_sent = False
+
 
     def start(self):
         """ Begin execution of this task. """
@@ -602,6 +669,18 @@ class Task(object):
         """ Runs completion flow for this task if it's finished. """
 
         if self.process.poll() is None:
+
+            # timeout check
+            if (self.soft_timeout != 0 and
+                (datetime.utcnow() - self.started_at).seconds >= self.soft_timeout and
+                not self.terminate_sent):
+                self.terminate()
+
+            if (self.hard_timeout != 0 and
+                (datetime.utcnow() - self.started_at).seconds >= self.hard_timeout and
+                not self.kill_sent):
+                self.kill()
+
             self._start_check_timer()
             return
 
@@ -610,6 +689,11 @@ class Task(object):
         for temp_file in [self.stdout_file, self.stderr_file]:
             temp_file.close()
 
+        if self.terminate_sent:
+            self.stderr += '\nDAGOBAH SENT SIGTERM TO THIS PROCESS\n'
+        if self.kill_sent:
+            self.stderr += '\nDAGOBAH SENT SIGKILL TO THIS PROCESS\n'
+
         self.stdout_file = None
         self.stderr_file = None
 
@@ -624,6 +708,7 @@ class Task(object):
         """ Send SIGTERM to the task's process. """
         if not self.process:
             raise DagobahError('task does not have a running process')
+        self.terminate_sent = True
         self.process.terminate()
 
 
@@ -631,6 +716,7 @@ class Task(object):
         """ Send SIGKILL to the task's process. """
         if not self.process:
             raise DagobahError('task does not have a running process')
+        self.kill_sent = True
         self.process.kill()
 
 
@@ -753,14 +839,16 @@ class Task(object):
         self.parent_job._complete_task(self.name, **kwargs)
 
 
-    def _serialize(self, include_run_logs=False):
+    def _serialize(self, include_run_logs=False, strict_json=False):
         """ Serialize a representation of this Task to a Python dict. """
 
         result = {'command': self.command,
                   'name': self.name,
                   'started_at': self.started_at,
                   'completed_at': self.completed_at,
-                  'success': self.successful}
+                  'success': self.successful,
+                  'soft_timeout': self.soft_timeout,
+                  'hard_timeout': self.hard_timeout}
 
         if include_run_logs:
             last_run = self.backend.get_latest_run_log(self.parent_job.job_id,
@@ -770,4 +858,6 @@ class Task(object):
                 if run_log:
                     result['run_log'] = run_log
 
+        if strict_json:
+            result = json.loads(json.dumps(result, cls=StrictJSONEncoder))
         return result
